	list p=16f84
;**************************************************************
;*
;* 	Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 IR-Empfnger-Impulse
;*		1 NC
;*		2 NC
;*		3 NC
;*		4 NC
;*	PORTB:	0 Bild vor/zurck
;*		1 schrfer+
;*		2 schrfer-
;*		3 NC
;*		4 Empfangskontrolle
;*		5 NC
;*		6 NC
;*		7 NC
;*
;**************************************************************
;
; manfred2801 08/2003
;
; Dekodieren von Canon Fernbedienung WL-E80
; und Ansteuerung eines Diaprojektors
; Takt 4 MHz
; 

;**************************************************************
; Konfiguration des PICs
;
; bis 4 MHz: Power on Timer, kein Watchdog, XT-Oscillator
	__CONFIG	_PWRTE_ON & _WDT_OFF & _XT_OSC


;**************************************************************
; Includedatei fr den 16F84 einbinden

	#include <P16f84a.INC>

; Variablennamen vergeben

w_copy	Equ	0x20		; Backup fr Akkuregister
s_copy	Equ	0x21		; Backup fr Statusregister
loops	Equ	0x22		; Zhler fr Warteschleife 
loops2	Equ	0x23		; Zhler fr Warteschleife 
Byte1	Equ	0x24		; Empfangenes Byte 1
Byte2	Equ	0x25		; Empfangenes Byte 2
Byte3	Equ	0x26		; Empfangenes Byte 3
Byte4	Equ	0x27		; Empfangenes Byte 4
BCount	Equ	0x28		; Zhlt empfangene Bits
Lang0	Equ	0x29		; Wie lange war LOW
Sbyte	Equ	0x2A		; Schmierbyte
SameCmd Equ	0x2B		; Selber Befehl noch einmal 
Blinker Equ	0x2C		; Blinker im Interrupt

;**************************************************************
; los gehts mit dem Programm

	org 	0
	goto	Init	

;**************************************************************
; die Interuptserviceroutine

	org 	4 	
intvec	bcf	INTCON, GIE	; disable Interupt 

	movwf	w_copy		; w retten
	swapf	STATUS, w 	; STATUS retten
	movwf	s_copy		;

	movlw	D'176'		; 256-80=176 ((1MHz : 32 ): 80 = 390 Hz = 2560 s)
	movwf	TMR0		; Wenn innerhalb von 2560 s kein Wechsel von 0 auf 1
				; Dann von vorne beginnen
; Intrupt servic routine
Int_serv 

	movlw	B'00000000'	; alle Empangbytes lschen
	movwf	Byte1
	movwf	Byte2
	movwf	Byte3
	movwf	Byte4
	movwf	BCount		;Bitcounter auch lschen
	movwf	Sbyte
	movlw	D'176'		; 256-80=176 ((1MHz : 32 ): 80 = 390 Hz = 2560 s)
	movwf	TMR0

	decf	SameCmd, 1	; Kommt das selbe Kommando nicht nochmal
	btfss	STATUS,Z 	; innerhalb von 100ms
	goto 	Int_weiter	; = 39 x 2560s
	bcf	PORTB,1		; dann Die betroffenen Ausgnge lschen
	bcf	PORTB,2

Int_weiter
	decf	Blinker, 1	; Blinker runterzhlen
	btfss	STATUS,Z 	; alle 500ms
	goto 	Int_end		; = 195 x 2560s
	movlw	D'195'		; 500ms laden
	movwf	Blinker 	; auf Blinker kopieren 
	goto 	Int_end


Int_end	swapf	s_copy, w	; STATUS zurck
	movwf	STATUS 
	swapf	w_copy, f	; w zurck mit flags
	swapf	w_copy, w

	bcf	INTCON, T0IF	; Interupt-Flag lschen
	bsf	INTCON, GIE	; enable Interupt

	retfie			; Return aus Interrupt


;**************************************************************
; Port A/B auf Eingabe/Ausgabe stellen

Init
	movlw	B'00000000'
	movwf	PORTA
	movwf	PORTB		; Anzeige dunkel
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'11111111'	; PortA alle input
	movwf	TRISA
	movlw	B'11110000'	; PortB alle output
	movwf	TRISB
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten
	movlw	D'176'		; 256-80=176 ((1MHz : 32 ): 80 = 390 Hz = 2560 s)
	movwf	TMR0
	movlw	D'195'		; 500ms laden
	movwf	Blinker 	; auf Blinker kopieren 

Init2
	clrf	Byte1
	clrf	Byte2
	clrf	Byte3
	clrf	Byte4
	clrf	BCount
	clrf	Lang0
	clrf	Sbyte

;**************************************************************
; Interrupt-Timer einstellen (Deadline)

	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'10000100'	; internen Takt zhlen, Vorteiler zum Timer0, 32:1
	movwf	OPTION_REG
	movlw	D'176'		; 256-80=176 ((1MHz : 32 ): 80 = 390 Hz = 2560 s)
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten
	movwf	TMR0
	bsf	INTCON, T0IE	; Timer0 interupt erlauben
	bsf	INTCON, GIE	; Interupt erlauben


;**********************************************************
; Pollen der Empfangsleitung

pollen

BISLOW
	btfsc	PORTA, 0
	goto	BISLOW		; warten auf low
	movlw	D'176'		; Interrupt rcksetzen
	movwf	TMR0
	movlw	D'0'		; LOW-Zeitmessung rcksetzen
	movwf	Lang0	

BISHIGH
	btfsc	PORTA, 0	; Ist HIGH ?
	goto	CHECKBIT	; Dann Bit checken
	call 	Wait156		; Ansonsten warten
	incf	Lang0, 1	; Merken wie lange auf LOW
	goto	BISHIGH		; und warten

CHECKBIT
	rlf	Sbyte, 1	; Schmierbyte nach links schieben
	incf	BCount, 1	; Bitcounter erhhen
	btfsc	Lang0, 3	; Bit3 gesetzt -> Zahl ist >=8 -> war 10x156s auf Log0
	bsf	Sbyte, 0	; das heisst LOG1
	btfss	Lang0, 3	; Bit3 nicht gesetzt -> Zahl ist <8 -> war 3x156s auf Log0 
	bcf	Sbyte, 0	; das heisst LOG0

B1
	Movf 	BCount,0	; 1.Byte vollstndig ?
	xorlw 	D'8'		; Bit8 ist angekommen 
	btfss	STATUS,Z	
	goto 	B2
	Movf	Sbyte, 0
	Movwf	Byte1		; Schmierbyte in echtes Byte kopieren
B2
	Movf 	BCount,0	; 2.Byte vollstndig ?
	xorlw 	D'16'		
	btfss	STATUS,Z	
	goto 	B3
	Movf	Sbyte, 0
	Movwf	Byte2		; Schmierbyte in echtes Byte kopieren
B3
	Movf 	BCount,0	; 3.Byte vollstndig ?
	xorlw 	D'24'		
	btfss	STATUS,Z	
	goto 	B4
	Movf	Sbyte, 0
	Movwf	Byte3	
B4
	Movf 	BCount,0	; 4. Byte vollstndig ?
	xorlw 	D'32'		
	btfss	STATUS,Z	
	goto 	B5
	Movf	Sbyte, 0
	Movwf	Byte4		; Schmierbyte in echtes Byte kopieren
	goto	Geschafft	; und ab zur Auswertung der 4 Bytes
B5
	movlw	D'176'		; Interrupt rcksetzen
	movwf	TMR0
	goto	BISLOW		; und auf nchstes Bit warten	

;**********************************************************
; Auswertung der empfangenen 4 Bytes

Geschafft

	Movf 	Byte1,0		; Byte1 ins Arbeitsregister
	xorlw 	B'11010000'	; XOR mit &A1
	btfss	STATUS,Z	; Sind die beiden gleich (Ergebnis=0)
	goto 	Init2		; War nix -> nochmal
	Movf 	Byte2,0		;
	xorlw 	B'10110111'	; Hier wird Byte 1 und 2 auf den Gertecode
	btfss	STATUS,Z	; geprft
	goto 	Init2		; Falscher Gertecode -> alles rcksetzen

T1
	Movf 	Byte3,0		; REC-Taste abfragen
	xorlw 	B'01110000'
	btfss	STATUS,Z
	goto 	T2
	Movf 	Byte4,0
	xorlw 	B'00001111'
	btfss	STATUS,Z
	goto 	T2
	goto	REC
T2
	Movf 	Byte3,0		; PLAY-Taste abfragen
	xorlw 	B'01000000'
	btfss	STATUS,Z
	goto 	T3
	Movf 	Byte4,0
	xorlw 	B'00111111'
	btfss	STATUS,Z
	goto 	T3
	goto	PLAY
T3
	Movf 	Byte3,0		; TELE-Taste abfragen
	xorlw 	B'00011100'
	btfss	STATUS,Z
	goto 	T4
	Movf 	Byte4,0
	xorlw 	B'01100011'
	btfss	STATUS,Z
	goto 	T4
	goto	TELE
T4
	Movf 	Byte3,0		; WIDE-Taste abfragen
	xorlw 	B'01011100'
	btfss	STATUS,Z
	goto 	T5
	Movf 	Byte4,0
	xorlw 	B'00100011'
	btfss	STATUS,Z
	goto 	T5
	goto	WIDE

T5
	goto	Init2		; Zurck zum Start


;**********************************************************
; Ausgnge beschalten


REC
	bsf	PORTB,0		; Bild vorwrts
	call	Wait125		; RB0 muss kurz anstehen
	bcf	PORTB,0
	call 	Wait125		; Dass sich das nicht gleich wiederholt
	goto	Init2
PLAY
	bsf	PORTB,0		; Bild vorwrts
	call	Wait125		; RB0 muss lnger anstehen
	call	Wait125
	call	Wait125
	call	Wait125
	call	Wait125
	bcf	PORTB,0
	call	Wait125		; Dass sich das nicht gleich wiederholt
	goto	Init2
TELE
	bcf	PORTB,2		; Rcksetzen des Schrfer -
	bsf	PORTB,1		; setzen des Schrfer + 
	movlw	D'39'		; Vielleicht kommt ja selber Befehl
	movwf	SameCmd		; Nochmal 
	goto	Init2
WIDE
	bcf	PORTB,1		; Rcksetzen des Schrfer +
	bsf	PORTB,2		; setzen des Schrfer - 
	movlw	D'39'		; Vielleicht kommt ja selber Befehl
	movwf	SameCmd		; Nochmal 
	goto	Init2


	goto	Init2



;**********************************************************
; Warteschleife fr 156s

Wait156
	movlw   .39           	; Zeitkonstante fr 156s (.39) x 4 Befehle
        movwf   loops2
Wai5    nop                    	; 
        decfsz  loops2, F      	; 156/39 s vorbei?
        goto    Wai5		; nein, noch nicht
	retlw	0           	; Rcksprung


;**********************************************************
; Warteschleife 125 ms

Wait125
	movlw	D'125'		; 125 ms Pause (D'125')
	movwf	loops	
Wai
	movlw   .110           ; Zeitkonstante fr 1ms (.110)
        movwf   loops2
Wai2    nop                    ; 
        nop
        nop
        nop
        nop
        nop
        decfsz  loops2, F      ; 1 ms vorbei?
        goto    Wai2           ; nein, noch nicht
                               ;
        decfsz  loops, F       ; 125 ms vorbei?
        goto    Wai            ; nein, noch nicht
        retlw   0              ; das Warten hat ein Ende


;**********************************************************
; Programm Ende

	end			; Gesamtes Programm Ende

